require("Atlas-Addon-Helpers.AtlasHelpers");
luanet.load_assembly("log4net");

local types = {}
types["log4net.LogManager"] = luanet.import_type("log4net.LogManager");
local log = types["log4net.LogManager"].GetLogger("AtlasSystems.Addons.AutomaticCancellation");

local Settings = {};
Settings.NVTGC = GetSetting("NVTGC");
Settings.MonitorQueues = GetSetting("MonitorQueues");
Settings.CancellationReason = GetSetting("CancellationReason");
Settings.EmailTemplateName = GetSetting("EmailTemplateName");

function Init()
    log:Debug("Initializing Cancellation addon.");
    InitializeSettings();
    RegisterSystemEventHandler("SystemTimerElapsed", "CheckMonitoredQueues");
end

function InitializeSettings()
    local nvtgcs = {};
    for _, nvtgc in ipairs(AtlasHelpers.StringSplit(",", Settings.NVTGC)) do
        nvtgc = AtlasHelpers.Trim(nvtgc);
        if nvtgc ~= "" then
            table.insert(nvtgcs, nvtgc);
        end
    end
    Settings.NVTGC = nvtgcs;

    local queues = {};
    for _, queue in ipairs(AtlasHelpers.StringSplit(",", Settings.MonitorQueues)) do
        queue = AtlasHelpers.Trim(queue);
        if queue ~= "" then
            table.insert(queues, queue);
        end
    end
    Settings.MonitorQueues = queues;

    Settings.EmailTemplateName = AtlasHelpers.Trim(Settings.EmailTemplateName);
    Settings.CancellationReason = AtlasHelpers.Trim(Settings.CancellationReason);
end

function CheckMonitoredQueues()
    for _, queue in ipairs(Settings.MonitorQueues) do
        local queueName = AtlasHelpers.Trim(queue);
        if queueName ~= "" then
            log:DebugFormat("Cancelling all Requests in queue: {0}", queueName);
            ProcessDataContexts("TransactionStatus", queueName, "CancelRequest");
        end
    end
end

function CancelRequest()
    local tn = tonumber(GetFieldValue("Transaction", "TransactionNumber"));
	local nvtgc = GetFieldValue("User", "NVTGC");

	if (not ShouldHandleRequest(nvtgc)) then
        log:InfoFormat("Skipping processing transaction {0}. NVTGC ({1}) is not configured for processing.", tn, nvtgc);
		return;
	end

    if (Settings.CancellationReason and Settings.CancellationReason ~= "") then
        log:Debug("Setting request's reason for cancellation.");
        SetFieldValue("Transaction", "ReasonForCancellation", Settings.CancellationReason);
        SaveDataSource("Transaction");
    end

    local emailTemplateName = Settings.EmailTemplateName;
    if (not emailTemplateName or emailTemplateName == "") then
        emailTemplateName = GetFieldValue("Transaction", "ProcessType") .. " Cancellation " .. GetFieldValue("Transaction", "RequestType");
    end

    log:DebugFormat("Sending email template {0} for transaction {1}", emailTemplateName, tn);
    ExecuteCommand("SendTransactionNotification", {tn, emailTemplateName});

    local cancellationQueue = "Cancelled by ILL Staff";
    log:DebugFormat("Routing transaction {0} to {1}", tn, cancellationQueue);
    ExecuteCommand("Route", {tn, cancellationQueue});
end

function ShouldHandleRequest(userNvtgc)
	if (#Settings.NVTGC > 0) then
        log:Debug("Checking if the request should be processed for  NVTGC (" .. userNvtgc .. ")");

        for index, nvtgc in ipairs(Settings.NVTGC) do
            if (nvtgc:lower() == userNvtgc:lower()) then
                return true;
            end
		end

		return false;
	else
		--Handle all requests because NVTGC is not configured		
		return true;
	end
end
